<?php
/**
 * The file that defines the core plugin class.
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://www.solwininfotech.com/
 * @since      1.0.0
 *
 * @package    Ualp_Google_Sheets_API
 * @subpackage Ualp_Google_Sheets_API/includes
 */

// Include Autoload file for Google Sheets API.
require_once plugin_dir_path( __FILE__ ) . 'vendor/autoload.php';
use \Firebase\JWT\JWT;
/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Ualp_Google_Sheets_API
 * @subpackage Ualp_Google_Sheets_API/includes
 * @author     Solwin Infotech <info@solwininfotech.com>
 */
class Ualp_Google_Sheets_API {

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since  1.0.0
	 * @access protected
	 * @var    string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;
	/**
	 * The current version of the plugin.
	 *
	 * @since  1.0.0
	 * @access protected
	 * @var    string    $version    The current version of the plugin.
	 */
	protected $version;
	/**
	 * Private_key Google Service Account Credentials
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $private_key    Private_key Google Service Account Credentials  .
	 */
	public $private_key;

	/**
	 * Google Service Account Credentials  client_email aka service account email
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $client_email    Google Service Account Credentials  client_email aka service account email
	 */
	public $client_email;

	/**
	 * Class Cunstructor.
	 */
	public function __construct() {

	}

	/**
	 * Creating google API tokens & Getting tokens from Google
	 *
	 * @param string $private_key Private Key.
	 * @param string $client_email Client Email.
	 */
	public static function ualp_generating_token_credential( $private_key, $client_email ) {
		$credential = get_option( 'ualpAddJsonCredentials' );
		$credential = json_decode( $credential );
		$payload    = array(
			'iss'   => $credential->client_email,
			'scope' => 'https://www.googleapis.com/auth/drive',
			'aud'   => 'https://oauth2.googleapis.com/token',
			'exp'   => time() + 3600,
			'iat'   => time(),
		);

		$jwt = JWT::encode( $payload, $credential->private_key, 'RS256' );

		$args    = array(
			'headers' => array(),
			'body'    => array(
				'grant_type' => 'urn:ietf:params:oauth:grant-type:jwt-bearer',
				'assertion'  => $jwt,
			),
		);
		$returns = wp_remote_post( 'https://oauth2.googleapis.com/token', $args );
		if ( isset( $_POST['btngooglesheetsAPI'] ) && isset( $_POST['_wp_google_sheets_api_setting_nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['_wp_google_sheets_api_setting_nonce'] ) ), '_wp_google_sheets_api_setting_action' ) ) {
			if ( is_wp_error( $returns ) || ! is_array( $returns ) || ! isset( $returns['body'] ) ) {
				$message = esc_html__( 'Token is not created!', 'user_activity_log_pro' );
				ualp_admin_notice_message( 'error', $message );
			} else {
				$message = esc_html__( 'Successfully Token Created.', 'user_activity_log_pro' );
				ualp_admin_notice_message( 'updated', $message );
			}
		}
		return json_decode( $returns['body'] );
	}

	/**
	 * Google Token.
	 */
	public static function ualp_google_token() {
		$google_token = get_option( 'ualp_google_token' );
		if ( $google_token && time() > $google_token->expires_in ) {
			if ( isset( $private_key ) && isset( $client_email ) ) {
				$new_token = self::ualp_generating_token_credential( $private_key, $client_email );
				if ( $new_token->access_token ) {
					$new_token->expires_in = time() + $new_token->expires_in;
					$google_token          = $new_token->access_token;
					update_option( 'ualp_google_token', $new_token );
				} else {
					$message = esc_html__( 'Please enter the credentials in JSON format!', 'user_activity_log_pro' );
					ualp_admin_notice_message( 'error', $message );
					return false;
				}
			}
		}
		return $google_token;
	}

	/**
	 * Spreadsheets.
	 */
	public static function ualp_spreadsheets() {
		$returns = ( isset( self::ualp_google_token()->access_token ) ) ? wp_remote_get( 'https://www.googleapis.com/drive/v3/files?access_token=' . self::ualp_google_token()->access_token ) : false;
		if ( is_wp_error( $returns ) || ! isset( $returns['response']['code'] ) || 200 != $returns['response']['code'] && isset( $_POST['btngooglesheetsAPI'] ) && isset( $_POST['_wp_google_sheets_api_setting_nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['_wp_google_sheets_api_setting_nonce'] ) ), '_wp_google_sheets_api_setting_action' ) ) {
			$message = esc_html__( 'Failed to return spreadsheets.', 'user_activity_log_pro' );
			ualp_admin_notice_message( 'error', $message );
			return wp_json_encode( $returns );
		}

		$spreadsheets = array();
		$body         = json_decode( $returns['body'], true );
		if ( isset( $body['files'] ) ) {
			$files = $body['files'];
			foreach ( $files as $file ) {
				if ( 'application/vnd.google-apps.spreadsheet' == $file['mimeType'] ) {
					$spreadsheets[ $file['id'] ] = $file['name'];
				}
			}
		}
		return $spreadsheets;
	}

	/**
	 * Worksheets
	 *
	 * @param string $spreadsheet_id Spreadsheet ID.
	 */
	public static function ualp_worksheets( $spreadsheet_id = '' ) {
		if ( ! is_string( $spreadsheet_id ) ) {
			$message = esc_html__( 'Spreadsheet ID is not string', 'user_activity_log_pro' );
			ualp_admin_notice_message( 'error', $message );
		}
		if ( empty( $spreadsheet_id ) ) {
			$message = esc_html__( 'Spreadsheet ID is empty', 'user_activity_log_pro' );
			ualp_admin_notice_message( 'error', $message );
		}

		$returns = ( isset( self::ualp_google_token()->access_token ) ) ? wp_remote_get( 'https://sheets.googleapis.com/v4/spreadsheets/' . $spreadsheet_id . '/?access_token=' . self::ualp_google_token()->access_token ) : false;

		if ( is_wp_error( $returns ) || ! isset( $returns['response']['code'] ) || 200 != $returns['response']['code'] ) {
			$message = esc_html__( 'Error on getting spreadsheets.', 'user_activity_log_pro' );
			ualp_admin_notice_message( 'error', $message );
			return wp_json_encode( $returns );
		}

		$sheets = array();
		$body   = json_decode( $returns['body'], true );
		foreach ( $body['sheets'] as $value ) {
			$sheets[ $value['properties']['sheetId'] ] = $value['properties']['title'];
		}
		return $sheets;
	}

	/**
	 * Spreadsheets and Worksheets
	 */
	public static function ualp_spreadsheets_and_worksheets() {
		$returns = ( isset( self::ualp_google_token()->access_token ) ) ? wp_remote_get( 'https://www.googleapis.com/drive/v3/files?access_token=' . self::ualp_google_token()->access_token ) : false;
		if ( isset( $_POST['btngooglesheetsAPI'] ) && isset( $_POST['_wp_google_sheets_api_setting_nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['_wp_google_sheets_api_setting_nonce'] ) ), '_wp_google_sheets_api_setting_action' ) ) {
			if ( is_wp_error( $returns ) || ! isset( $returns['response']['code'] ) || 200 != $returns['response']['code'] ) {
				$message = esc_html__( 'Error on getting shared spreadsheets.', 'user_activity_log_pro' );
				ualp_admin_notice_message( 'error', $message );
				return wp_json_encode( $returns );
			}
		}
		$body = json_decode( $returns['body'], true );
		if ( isset( $body['files'] ) ) {
			$files                  = $body['files'];
			$spreadsheets           = array();
			$spreadsheets_worksheet = array();
			foreach ( $files as $file ) {
				if ( 'application/vnd.google-apps.spreadsheet' == $file['mimeType'] ) {
					$spreadsheets[ $file['id'] ] = $file['name'];
				}
			}
			foreach ( $spreadsheets as $spreadsheets_key => $spreadsheets_name ) {
				$worksheets_returns = ( isset( self::ualp_google_token()->access_token ) ) ? wp_remote_get( 'https://sheets.googleapis.com/v4/spreadsheets/' . $spreadsheets_key . '/?access_token=' . self::ualp_google_token()->access_token ) : false;
				if ( ! is_wp_error( $worksheets_returns ) && isset( $worksheets_returns['response']['code'] ) && 200 == $worksheets_returns['response']['code'] ) {
					$worksheets_response_body = json_decode( $worksheets_returns['body'], true );
					$sheets                   = array();
					foreach ( $worksheets_response_body['sheets'] as $value ) {
						$sheets[ $value['properties']['sheetId'] ] = $value['properties']['title'];
					}
					$spreadsheets_worksheet[ $spreadsheets_key ] = array( $spreadsheets_name, $sheets );
				} else {
					$message = esc_html__( 'Error on getting shared spreadsheets and worksheets.', 'user_activity_log_pro' );
					ualp_admin_notice_message( 'error', $message );
					return wp_json_encode( $worksheets_returns );
				}
			}
			return $spreadsheets_worksheet;
		}
	}

	/**
	 * Insert data into Google  spreadsheets
	 *
	 * @param string $spreadsheet_id     Google Spreadsheet ID.
	 * @param string $worksheets_id      Google worksheet ID.
	 * @param array  $data_array         Data array.
	 */
	public static function ualp_append_row( $spreadsheet_id = '', $worksheets_id = '', $data_array = '' ) {

		$worksheets                     = self::ualp_worksheets( $spreadsheet_id );
		$ualp_spreadsheet_and_worksheet = get_option( 'ualpSpreadsheetAndWorksheet' );
		if ( $worksheets[0] ) {
			$worksheet_name = $ualp_spreadsheet_and_worksheet;
		} else {
			$message = esc_html__( 'Error in getting worksheets.', 'user_activity_log_pro' );
			ualp_admin_notice_message( 'error', $message );
			return wp_json_encode( $worksheets );
		}
		$url    = 'https://sheets.googleapis.com/v4/spreadsheets/' . $spreadsheet_id . '/values/' . $worksheet_name . '!A:A:append?valueInputOption=USER_ENTERED';
		$args   = array(
			'headers' => array(
				'Authorization' => 'Bearer ' . self::ualp_google_token()->access_token,
				'Content-Type'  => 'application/json',
			),
			'body'    => '{"range":"' . $worksheet_name . '!A:A", "majorDimension":"ROWS", "values":[' . wp_json_encode( array_values( $data_array ) ) . ']}',
		);
		$return = wp_remote_post( $url, $args );
		if ( isset( $_POST['btngooglesheetsAPI'] ) && isset( $_POST['_wp_google_sheets_api_setting_nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['_wp_google_sheets_api_setting_nonce'] ) ), '_wp_google_sheets_api_setting_action' ) ) {
			if ( ! is_wp_error( $return ) ) {
				$message = esc_html__( 'Successfully added the log to the sheet.', 'user_activity_log_pro' );
				ualp_admin_notice_message( 'updated', $message );
				return wp_json_encode( array_values( $data_array ) );
			} else {
				$message = esc_html__( 'Error while adding the log to the sheet.', 'user_activity_log_pro' );
				ualp_admin_notice_message( 'error', $message );
				return wp_json_encode( $return );
			}
		}
	}
}
